<?xml version="1.0"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text" indent="yes" />

  <!--  props.xsl
        Rick Landau, Dell, 20070212
        From Projector&Display Mgt WG XML for mgt group properties, 
        extract a fragment of SNMP MIB syntax. 
        
        In this case, extract the OBJECT-TYPE definition for each property.  
        -->

  <xsl:variable name="SNMPprefix">pdm</xsl:variable>
  <xsl:variable name="SNMPPrefix">Pdm</xsl:variable>
  <xsl:variable name="MIBname">pdmMIBObjects</xsl:variable>

  <!-- G r o u p -->
  <xsl:template match="Group">
<!--    - seen:Group <xsl:value-of select="@GroupName"/>\n-->
    -- Processed by props14.xsl at   .  .  .  <!-- <xsl:value-of select="xsl:current-dateTime()"/> -->

<xsl:value-of select="$SNMPprefix"/><xsl:value-of select="@GroupName"/> OBJECT IDENTIFIER { <xsl:value-of select="$MIBname"/> inserthere-index }  
    <xsl:apply-templates select="Property">
      <xsl:with-param name="GroupName"><xsl:value-of select="@GroupName"/></xsl:with-param>
    </xsl:apply-templates>
    <xsl:apply-templates select="Table">
      <xsl:with-param name="GroupName"><xsl:value-of select="@GroupName"/></xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>
  
  <!-- T a b l e -->
  <xsl:template name="Table" match="Table">
    <xsl:param name="GroupName"/>
<!--    - seen:Table <xsl:value-of select="@TableName"/>\n-->

    <xsl:text>
</xsl:text>
    --<xsl:value-of select="$SNMPPrefix"/><xsl:value-of select="@TableName"/>Entry ::= SEQUENCE {
    --EDITMEOUT EDITMEOUT EDITMEOUT EDITMEOUT EDITMEOUT EDITMEOUT EDITMEOUT
    -- table at { <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$GroupName"/><xsl:text> </xsl:text><xsl:value-of select="position()"/> }
    --}
<!--
 --><xsl:variable name="myentryis">
      <xsl:value-of select="$SNMPprefix"/>
      <xsl:value-of select="@TableName"/>
      <xsl:if test="not(contains(@TableName,'Entry'))">Entry</xsl:if>
    </xsl:variable>

<!-- Index property that goes at the beginning of every table -->
<xsl:value-of select="$SNMPprefix"/>
    <xsl:value-of select="$GroupName"/>Index OBJECT-TYPE
    SYNTAX     Integer32 (1..65535)
    MAX-ACCESS not-accessible
    STATUS     current
    DESCRIPTION
        "A unique value used by the device to identify this
        entry.  Values are expected to remain
        stable across successive power cycles."
    ::= { <xsl:value-of select="$myentryis"/> 1 }

<!-- Find all Properties, inside or outside tables. -->
    <xsl:apply-templates select="Property">
      <xsl:with-param name="GroupName"><xsl:value-of select="$GroupName"/></xsl:with-param>
      <xsl:with-param name="TableName">
        <xsl:value-of select="@TableName"/>
        <xsl:if test="not(contains(@TableName,'Table'))">Table</xsl:if>
      </xsl:with-param>
      <xsl:with-param name="EntryName">
        <xsl:value-of select="@TableName"/>
        <xsl:if test="not(contains(@TableName,'Entry'))">Entry</xsl:if>
      </xsl:with-param>
    </xsl:apply-templates>
  </xsl:template>

  <!-- P r o p e r t y -->
  <xsl:template name="Property" match="Property">
    <xsl:param name="GroupName"/>
    <xsl:param name="TableName"/>
    <xsl:param name="EntryName"/>

<!--    - Property at-AbstractName=<xsl:value-of select="@AbstractName"/>
    - parent is type=<xsl:value-of select="name(..)"/>
    - Property in table at-TableName=<xsl:value-of select="../@TableName"/>
    - Property in group at-name=<xsl:value-of select="../../@name"/>\n-->
    <xsl:variable name="mynameis">
      <xsl:choose>
        <xsl:when test="contains(SNMPName,' ')">
          <xsl:value-of select="$SNMPprefix"/>
          <xsl:value-of select="$GroupName"/>
          <xsl:value-of select="@AbstractName" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$SNMPprefix"/>
          <xsl:value-of select="SNMPName" />
        </xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>
    
    <!-- What is the right level of context for this property? -->
    <!-- If this is a table column, position it within the table.
    Otherwise, position it within the containing group. -->
    <xsl:variable name="mycontextis">
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$GroupName" />
        </xsl:when>
        <xsl:when test="name(..)='Table'">
          <xsl:value-of select="$SNMPprefix"/><xsl:value-of select="$EntryName" />
        </xsl:when>
        <xsl:otherwise>ErrorUnknownContext</xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>

    <!-- What is the right numbering at this level in the OID for this property? -->
    <!-- Numbering inside tables is offset by one because of
         the addition of the Index property into the table. -->
    <xsl:variable name="myoidlevelnumberis">
      <xsl:choose>
        <xsl:when test="name(..)='Group'">
          <xsl:value-of select="position()"/>
        </xsl:when>
        <xsl:when test="name(..)='Table'">
          <xsl:value-of select="position()+1"/>
        </xsl:when>
        <xsl:otherwise>ErrorUnknownContext</xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>

    <!-- What is the matching data syntax in SNMP? -->
    <xsl:variable name="mysyntaxis">
      <xsl:choose>
        <!-- enum -->
        <xsl:when test="Datatype/@Type='EnumeratedInteger'"><xsl:value-of select="$mynameis" />TC  -- Enum</xsl:when>
        <!-- integer, maybe with range -->
        <xsl:when test="Datatype/@Type='Integer'"><xsl:text>Integer32 </xsl:text>
            <xsl:if test="Datatype/RangeMin|Datatype/RangeMax">
                <xsl:text>(</xsl:text>
                <xsl:value-of select="Datatype/RangeMin"/>..<xsl:value-of select="Datatype/RangeMax"/>
                <xsl:text>)</xsl:text> -- Integer<!--
            --></xsl:if><!--
        --></xsl:when>
        <!-- gauge, with range -->
        <xsl:when test="Datatype/@Type='Gauge'"><xsl:text>Integer32 </xsl:text>
            <xsl:if test="Datatype/RangeMin|Datatype/RangeMax">
                <xsl:text>(</xsl:text>
                <xsl:value-of select="Datatype/RangeMin"/>..<xsl:value-of select="Datatype/RangeMax"/>
                <xsl:text>)</xsl:text><xsl:text>  -- Gauge</xsl:text>
            </xsl:if>
        </xsl:when>
        <!-- counter -->
        <xsl:when test="Datatype/@Type='Counter'">Integer32  -- Counter</xsl:when>
        <!-- integer constant -->
        <xsl:when test="Datatype/@Type='IntegerConstant'"><xsl:text>Integer32</xsl:text></xsl:when>
        <!-- string, with length -->
        <xsl:when test="Datatype/@Type='String'"><xsl:text>OCTET STRING </xsl:text>
            <xsl:if test="Datatype/LengthMin|Datatype/LengthMax"><xsl:text>(SIZE(</xsl:text></xsl:if>
            <xsl:value-of select="Datatype/LengthMin"/>..<xsl:value-of select="Datatype/LengthMax"/>
            <xsl:if test="Datatype/LengthMin|Datatype/LengthMax"><xsl:text>))</xsl:text></xsl:if>
            <xsl:text>  -- String</xsl:text>
        </xsl:when>
        <!-- string constant, with length -->
        <xsl:when test="Datatype/@Type='StringConstant'"><xsl:text>OCTET STRING  -- StringConstant</xsl:text>
            <xsl:if test="Datatype/LengthMin|Datatype/LengthMax"><xsl:text>(SIZE(</xsl:text></xsl:if>
            <xsl:value-of select="Datatype/LengthMin"/>..<xsl:value-of select="Datatype/LengthMax"/>
            <xsl:if test="Datatype/LengthMin|Datatype/LengthMax"><xsl:text>))</xsl:text></xsl:if>
            <xsl:text>  -- StringConstant</xsl:text>
        </xsl:when>
        <!-- boolean -->
        <xsl:when test="Datatype/@Type='Boolean'"><xsl:text>TruthValue  -- Boolean, import from SMIv2-TC</xsl:text>
        </xsl:when>
        <!-- what else do we use? -->
        <!-- add other datatype translations in here -->
        <!-- unrecognized (yet) -->
        <xsl:otherwise>ErrorUnknownDatatype</xsl:otherwise>
      </xsl:choose> 
    </xsl:variable>
    <xsl:text>
</xsl:text>

<!-- What is the description for this property? -->
<!-- Clean up white space and remove double quotes because the
     SNMP DESCRIPTION clause is quoted. -->
    <xsl:variable name="mydescriptionis2">
      <xsl:value-of select="normalize-space(Definition)"/>
    </xsl:variable>
    <xsl:variable name="mydescriptionis">
      <xsl:value-of select="translate($mydescriptionis2,'&quot;',&quot;&apos;&quot;)"/>
    </xsl:variable>

<!-- What is the default value spec for this property? -->
<!-- Depends a lot on the datatype. -->
    <xsl:variable name="mydefvalis2">
    <xsl:if test="contains(DefaultValue/Present,'Y')">
      <xsl:choose>
        <!-- enum -->
        <xsl:when test="Datatype/@Type='EnumeratedInteger'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- integer, maybe with range -->
        <xsl:when test="Datatype/@Type='Integer'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- gauge, with range -->
        <xsl:when test="Datatype/@Type='Gauge'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- counter -->
        <xsl:when test="Datatype/@Type='Counter'"><xsl:value-of select="DefaultValue/Value"/><!--
        <!xx Counters not allowed to have DEFVAL xx>
        --></xsl:when>
        <!-- integer constant -->
        <xsl:when test="Datatype/@Type='IntegerConstant'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- string, with length -->
        <xsl:when test="Datatype/@Type='String'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- string constant, with length -->
        <xsl:when test="Datatype/@Type='StringConstant'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- boolean -->
        <xsl:when test="Datatype/@Type='Boolean'"><xsl:value-of select="DefaultValue/Value"/><!--
        --></xsl:when>
        <!-- what else do we use? -->
        <!-- add other datatype translations in here -->
        <!-- unrecognized (yet) -->
        <xsl:otherwise>ErrorUnknownDatatype</xsl:otherwise>
      </xsl:choose> 
    </xsl:if>
    </xsl:variable>
    <!-- But if there is no default value specified at all, then 
            try to fake an appropriate value for the datatype. -->
    <xsl:variable name="mydefvalis">
    <xsl:if test="$mydefvalis2">
      <xsl:value-of select="$mydefvalis2"/>
    </xsl:if>
    <xsl:if test="not($mydefvalis2)">
      <xsl:choose>
        <!-- integer, maybe with range -->
        <xsl:when test="Datatype/@Type='Integer'">0<!--
        --></xsl:when>
        <!-- gauge, with range -->
        <xsl:when test="Datatype/@Type='Gauge'">0<!--
        --></xsl:when>
        <!-- integer constant -->
        <xsl:when test="Datatype/@Type='IntegerConstant'">0<!--
        --></xsl:when>
        <!-- string, with length -->
        <xsl:when test="Datatype/@Type='String'">""<!--
        --></xsl:when>
        <!-- string constant, with length -->
        <xsl:when test="Datatype/@Type='StringConstant'">""<!--
        --></xsl:when>
      </xsl:choose>
    </xsl:if>
    </xsl:variable>
    
<!-- Finally, generate the OBJECT-TYPE definition for the Property. -->
<xsl:value-of select="$mynameis"/> OBJECT-TYPE
    SYNTAX <xsl:value-of select="$mysyntaxis" />
    MAX-ACCESS <xsl:choose>
      <xsl:when test="contains(UserWritable,'Yes')">read-write</xsl:when>
      <xsl:otherwise>read-only</xsl:otherwise>
      </xsl:choose>
    <xsl:if test="not(Datatype/@Type='Counter')">
    DEFVAL <xsl:value-of select='$mydefvalis'/></xsl:if>
    STATUS current
    DESCRIPTION 
      <xsl:text>"</xsl:text><xsl:value-of select="$mydescriptionis"/><xsl:text>"</xsl:text>
    ::= { <xsl:value-of select="$mycontextis"/><xsl:text> </xsl:text>
        <xsl:value-of select="$myoidlevelnumberis"/> }
  </xsl:template><!--end PROPERTY-->

</xsl:stylesheet>

<!--
Edit history:
20070212    RBL v07 Original version.
20070310    RBL v10 Usable version.
20070909    RBL v12 Changed naming and numbering of scalar and table
                    properties per comments from Ira McDonald on 
                    proper SNMP naming.  
                    Add datatypes; still not complete.
20070910    RBL v13 Fix formatting for improved legibility.  
20070913    RBL v14 Add Index property at the beginning of table.

-->

